#!/bin/perl -w

# Define script configuration constants.

  $| = 1;   # Force output to be flushed immediately, no buffering.

  $bpnbat = '/usr/openv/netbackup/bin/bpnbat';
  $bpnbaz = '/usr/openv/netbackup/bin/admincmd/bpnbaz';

  @rapid_install_steps = qw {pre_install nbat_local_machine_add nbat_local_machine_login
         nbaz_setup_sec nbat_local_login nbaz_allow_auth 
         nbat_client_machine_add nbat_client_machine_login };


# Global working variables.

  $fatal_errors = 0;
  $step = "main";
  $result = -1;


# Configuration variables.

# NB Master system and file containing NB client system list are required
# arguments.
#
# If not supplied with optional argument values, the At Broker system and Az
# Server system values are initialized to the NB Master system value.

  $master_system = $at_broker_system = $az_server_system = "";
  @client_systems = ();

  $dhcp_host = "n";
  $at_broker_type = "unixpwd";
  $at_broker_port = "";  # null value accepts command's default port

  $default_machine_password = $security_admin_user = $security_admin_password = "";
# $default_machine_user is not used since the machine user is always the hostname.

#
# *** Customization Notes ***
#
#
# No user interface to support the functionality is externalized, however
# users proficient at customizing Perl scripts may choose to modify a copy of
# this unsupported script to initialize the associative array
# %machine_password in order to define VxSS machine password data specific to
# each host system specified as an NB Master, Media Serevr, or client system.
#
# The values provided will be used for creation of the VxSS machine
# certificate, and need not be the validation information for primary
# administrative user on the systems, nor use the password value defined for
# any existing operating system user.
#
# The processing logic of the script will check the associative array for
# machine password information using the hostname involved in each the
# processing steps and override the default machine password information if
# found.  The default machine password value itself is supplied as an argument
# to the script.
#
#
# The synatax for attempting such customization is:
#
# $machine_password{'hostname1.domainname.suffix'} = 'password1';
#
# ...or...
#
# $machine_password = {
#   'hostname1.domainname.suffix' => 'password1',
#   'hostname2.domainname.suffix' => 'password2',
#   'hostname3.domainname.suffix' => 'password3',
# }
#
#
# Similarily, the script as-shipped assumes a Broker Type of "unixpwd" and the
# default broker port are appropriate in all cases they are used, and that no
# DHCP hosts are involved.  The $at_broker_type, $at_broker_port, and
# $dhcp_host global variables are initialized to support the default mode, and
# would need additional code added to be set on a per-host basis.
#
# Users are advised not to attempt script customization if the information
# provided above seems unclear, is inconsistent with their understanding of
# Perl variable initialization, or use of the script is contrary to the advice
# of their security administrator.
#
# Users should be cautioned that the plaintext of any password values used in
# script customization are only protected by setting the appropriate file
# permissions on the resulting script to prevent unauthorized viewing.  Any
# user capable of reading the script will consequently have access to the
# plaintext password values conveniently listed by hostname.  Delivery of the
# customized script via e-mail or file transfer over insecure network links
# represents a potential security risk that should be understood prior to
# proceeding.


################################################################################
#                                                                              #
# usage() - Helper function to list usage syntax.                              #
#                                                                              #
################################################################################
sub usage {

 print <<__END_OF_USAGE_TEXT__
USAGE: $0 -help
USAGE: $0 -m NB_Master_Server -c client_list_file
       < prompted for authorization to continue >
       < prompted for machine password >
       < prompted for security admin user >
       < prompted for security admin password >
__END_OF_USAGE_TEXT__
}


################################################################################
#                                                                              #
# help() - Helper function to list detailed usage information.                 #
#                                                                              #
################################################################################
sub help {

  print <<__END_OF_HELP_TEXT__
$0:
GENERAL OVERVIEW

This script is provided to assist with deployment of NetBackup Access Control
configurations involving a large number of client systems by automating the
following steps:

  *  Initial NetBackup Access Control on the NetBackup Master Server.

  *  Addition of client systems to the list of systems allowed to use
     NetBackup Access Control with the Master Server.

  *  Definition of the NetBackup Access Control machine password values for
     the client systems.

  *  Remote execution of the machine login step on the client systems using
     rsh.

Appropriate NetBackup and VxSS installations must exist on the Master Server
and all client systems, and the client systems must be configured to accept
rsh requests from the Master Server in order for this script to function.

Please review the NetBackup Access Control documentation in the "Access
Management" section NetBackup System Administration Guide's for UNIX
prior to attempting configuration or execution of this script.


ARGUMENT DESCRIPTIONS

  -m  Specifies the preferred NetBackup Master Server name.  The
      Master Server name must be supplied.

  -c  Specifies the file containing the names of the NetBackup client
      systems that are to have initial NetBackup Access Control 
      configuration steps attempted.  The client systems filename
      must be supplied.


VALUES PROMPTED FOR AT EXECUTION TIME

  <machine certificate passord >
      Specifies the default machine password value to be used when the
      "bpbnat" command is used to create a machine credential file for
      each of the client systems.   The machine password value will be
      prompted for if not supplied.

  <security admin user>
      Specifies the user name to be used as the NetBackup Access Control
      security administrator.

  <security admin password>
      Specifies the password to be used as the NetBackup Access Control
      security administrator.

__END_OF_HELP_TEXT__

}


################################################################################
#                                                                              #
# process_arguments() - Helper function to list usage syntax.                  #
#                                                                              #
################################################################################
sub process_arguments {
  if (($#ARGV < 0) || (grep/\-usage|\-\?/, @ARGV)) {
    &usage();

    exit -1;

  } elsif (grep/\-help/, @ARGV) {
    &help();

    exit -1;
  }

  $index = 0;
  while ($index <= $#ARGV) {
    if ($ARGV[$index] eq "-m") {
      $index++;
      if ($ARGV[$index]) {
        $master_system = $ARGV[$index];

      } else {
        print "Argument \"-m\" provided, but no NB Master Server value found\n";
        &usage();

        exit -1;
      }

    } elsif ($ARGV[$index] eq "-c") {
      $index++;
      if ($ARGV[$index]) {
        $client_list_file = $ARGV[$index];
        die "Can not open client list file '$client_list_file'" if (! -f "$client_list_file");

        open(CLIENTS, "< $client_list_file");
        @client_systems = <CLIENTS>;
        while($_ = <CLIENTS>) {
          push(@client_systems, split(/ /, $_));
        }
        close(CLIENTS);
        chomp(@client_systems);

      } else {
        print "Argument \"-c\" provided, but no NB client system list filename found\n";
        &usage();

        exit -1;
      }
    }

    $index++;
  }
}


################################################################################
#                                                                              #
# $timestamp() - Helper function which returns the current date-time as a      #
#                string for progress messages.                                 #
#                                                                              #
################################################################################
$timestamp = sub {
  my($seconds, $minutes, $hours, $month_days, $month_number, $years) = localtime(time());

  return (sprintf("%02d:%02d:%02d: ", ($hours, $minutes, $seconds)));
};


################################################################################
#                                                                              #
# logmsg() - Helper function which prints progress messages.                   #
#                                                                              #
################################################################################
sub logmsg {
  my @lines = @_;
  my $line_index = 0;

  my $timestamp_length = 10;
  my $timestamp_indent = " " x $timestamp_length;

  for ($line_index=0; $line_index <= $#lines; $line_index++) {
    if ($line_index == 0) {
      print &$timestamp . $lines[$line_index] . "\n";
    } else {
      print $timestamp_indent . $lines[$line_index] . "\n";
    }
  }
}


################################################################################
#                                                                              #
# list_preconditions() - Informs the user that script execution is expected to #
#                        to take place on the NetBackup Master Server, and     #
#                        provides advice on script execution time.             #
#                                                                              #
################################################################################
sub list_preconditions {
  print <<__END_OF_RUN_TIME_NOTICES__

$0:
RUN-TIME NOTICES

This script is provided to assist with deployment of NetBackup Access Control
configurations involving a large number of client systems by automating the
inital steps documented in the Access Management section of the NetBackup
System Administrators Guide.

Execution of this script should take place on the NetBackup Master Server,
and will require a default machine password value, the username of the first
Security Administrator, and the password value for the first Security
Administrator to be provided interactively. 

If the current system is not the NetBackup Master Server, or if the
Security Administrator or machine password values required can not be
supplied in a secure manner, execution of this script can be halted at
the next interactive prompt.

The "bpnbaz -setupsecurity" command will be used during execution of this
script.  If run in an existing NetBackup Access Control configuration,
some end-user customization data may be overwritten.  Please review
the Access Management section of the NetBackup Administrator's Guide
prior to executing this script in an existing Access Control configuration.

The "rsh" command is used to facilitate command requests to remote NetBackup
Media Server and Client systems.  The root level ".rhosts" file on the remote
systems must be configured to allow remote execution of commands from the
NetBackup Master Server in order for this script to function.  Since network
requests to each host specified in the clients list file will be attempted,
execution of this script may require substantial time to complete when used
to configure a large number of hosts.

__END_OF_RUN_TIME_NOTICES__
}

################################################################################
#                                                                              #
# prompt_for_continue() - prompts for authorization to continue execution.     #
#                                                                              #
################################################################################
sub replace_shell_meta_chars {
  my $input = shift @_;

  $input =~ s/`/\\`/gx;
  $input =~ s/"/\\"/gx;

  return $input;
}

################################################################################
#                                                                              #
# prompt_for_continue() - prompts for authorization to continue execution.     #
#                                                                              #
################################################################################
sub prompt_for_continue {
  my $input = "";

  print "Continue initial NetBackup Access Control configuration? [n]: ";
  $input = <STDIN>;
  print "\n";

  if (uc(substr($input, 0, 1)) ne "Y") {
    print "Automated initial configuration canceled.\n";

    exit -1;

  } else {
    print "\n" x 2;
  }
}


################################################################################
#                                                                              #
# prompt_for_mach_pass() - prompts for the default machine password stored in  #
#                          the $default_machine_password global variable.      #
#                                                                              #
################################################################################
sub prompt_for_mach_pass {
  system("stty", "-echo");
  print "Enter the password value to be used for machine credentials: ";
  print " (echo suppressed): ";

  my $input = <STDIN>;
  chop($input);
  $default_machine_password = $input;
  $default_machine_password = &replace_shell_meta_chars($input);

  print "\n";
  system("stty", "echo");
}


################################################################################
#                                                                              #
# prompt_for_sec_admin_user() - prompts for the security administrator user    #
#                          name stored in the $security_admin_user global var. #
#                                                                              #
################################################################################
sub prompt_for_sec_admin_user {
  print "Enter the user name to be used for the first Security Administrator: ";

  my $input = <STDIN>;
  chop($input);
  $security_admin_user = $input;
}


################################################################################
#                                                                              #
# prompt_for_sec_admin_pass() - prompts for the security administrator user    #
#                          password stored in the $security_admin_password     #
#                          global variable.                                    #
#                                                                              #
################################################################################
sub prompt_for_sec_admin_pass {
  system("stty", "-echo");

  print "Enter the password value to be used for the Security Administrator ";
  print " (echo suppressed): ";

  my $input = <STDIN>;
  chop($input);
  $security_admin_password = $input;
  $security_admin_password =~ s/`/\\`/gx;
  $security_admin_password = &replace_shell_meta_chars($input);

  print "\n";

  system("stty", "echo");
}


################################################################################
#                                                                              #
# pre_install() - verify release materials are available to begin install &    #
#                 configuration of NetBackup Access Control configurations.    #
#                                                                              #
################################################################################
sub pre_install {
  my @required_files = ($bpnbat, $bpnbaz);
  my $file = "";

  logmsg("$step: Verifying required install materials.");

  foreach $file (@required_files) {
    if (! -x "$file") {
      $fatal_errors++;
      logmsg("$step: ERROR: file not found: $file");
    }
  }
}


################################################################################
#                                                                              #
# nbat_local_machine_add() - add the local machine to the set of machines      #
#                            using VxSS authentication.                        #
#                                                                              #
################################################################################
sub nbat_local_machine_add {
  my $password  = $machine_password{$master_system} ? $machine_password{$master_system} : $default_machine_password;

  logmsg("$step: Adding the local system to the", "list of systems using VxSS.");

  $result = system <<__END_OF_BPBNAT_ADDMACH_CALL__;
$bpnbat -addmachine  <<__END_OF_BPBNAT_ADDMACH__PARAMETERS__ > /dev/null
$dhcp_host
$at_broker_system
$at_broker_port
$master_system
$password
$password
__END_OF_BPBNAT_ADDMACH_PARAMETERS__
__END_OF_BPBNAT_ADDMACH_CALL__
  if ($result == 0) {
    logmsg("$step: successful add of master server", "$master_system");
  } else {
    $fatal_errors++;
    logmsg("$step: ERROR: Call to 'bpnbat' command failed: $result\n");
  }
}


################################################################################
#                                                                              #
# nbat_local_machine_login() - login the local machine in order to use VxSS    #
#                             authentication.                                  #
#                                                                              #
################################################################################
sub nbat_local_machine_login {
  my $password  = $machine_password{$master_system} ? $machine_password{$master_system} : $default_machine_password;

  logmsg("$step: Authenticating local system", "with VxSS.");

  $result = system <<__END_OF_BPBNAT_LOGIN_MACH_CALL__;
$bpnbat -loginmachine  <<__END_OF_BPBNAT_LOGIN_MACH_PARAMETERS__ > /dev/null
n
$at_broker_system
$at_broker_port
$master_system
$password
__END_OF_BPBNAT_LOGIN_MACH_PARAMETERS__
__END_OF_BPBNAT_LOGIN_MACH_CALL__
  if ($result == 0) {
    logmsg("$step: successful login of master server", "$master_system");
  } else {
    $fatal_errors++;
    logmsg("$step: ERROR: Call to 'bpnbat' command failed: $result");
  }
}


################################################################################
#                                                                              #
# nbaz_setup_sec() - Initialize the VxSS tables needed to perform NetBackup    #
#                    authorization.  Define the first Security Administrator   #
#                    identity.                                                 #
#                                                                              #
################################################################################
sub nbaz_setup_sec {
  my $user  = $security_admin_user;
  my $password  = $security_admin_password;

  logmsg("$step: Initializing Authorization database.", "(may require several minutes to complete).");

  $result = system <<__END_OF_BPNBAZ_SETUP_SEC_CALL__;
$bpnbaz -setupsecurity $master_system -server $az_server_system <<__END_OF_BPBNAZ_SETUP_SEC_PARAMETERS__ > /dev/null
$at_broker_system
$at_broker_port
$at_broker_type
$at_broker_system
$user
$password
__END_OF_BPNBAZ_SETUP_SEC_PARAMETERS__
__END_OF_BPNBAZ_SETUP_SEC_CALL__
  if ($result == 0) {
    logmsg("$step: setup security successful on master server", "$master_system");
  } else {
    $fatal_errors++;
    logmsg("$step: ERROR: Call to 'bpnbaz' command failed: $result");
  }
}


################################################################################
#                                                                              #
# nbat_local_login() - login the Security Administrator on the local system.   #
#                                                                              #
################################################################################
sub nbat_local_login {
  my $user  = $security_admin_user;
  my $password  = $security_admin_password;

  logmsg("$step: Initializing Authorization database.", "(may require several minutes to complete).");

  $result = system <<__END_OF_BPNBAT_LOGIN_CALL__;
$bpnbat -login <<__END_OF_BPBNAT_LOGIN_CALL__ > /dev/null
$at_broker_system
$at_broker_port
$at_broker_type
$at_broker_system
$user
$password
__END_OF_BPNBAT_LOGIN_PARAMETERS__
__END_OF_BPNBAT_LOGIN_CALL__
  if ($result == 0) {
    logmsg("$step: setup security successful on master server", "$master_system");
  } else {
    $fatal_errors++;
    logmsg("$step: ERROR: Call to 'bpnbaz' command failed: $result");
  }
}



################################################################################
#                                                                              #
# nbaz_allow_auth() - Enable authorization processing with the client systems. #
#                                                                              #
################################################################################
sub nbaz_allow_auth {
  logmsg("$step: Adding remote systems to the list of systems", "using VxSS.");

  foreach $client_system (@client_systems) {
    next if $client_system eq ""; 

    $result = system("$bpnbaz -allowauthorization $client_system > /dev/null");
    if ($result == 0) {
       logmsg("$step: added $client_system");
    }
    if ($result != 0) {
      $fatal_errors++;
      logmsg("$step: ERROR: Call to 'bpnbaz' command failed: $result");
    }
  }
}


################################################################################
#                                                                              #
# nbat_client_machine_add()  - Add the client systems to the list of systems   #
#                              with defined machine credentials.               #
#                                                                              #
################################################################################
sub nbat_client_machine_add {
  my $password = $machine_password{$client_system} ? $machine_password{$client_system} : $default_machine_password;

  logmsg("$step: Adding remote systems to the", "list of systems using VxSS.");

  foreach $client_system (@client_systems) {
    next if $client_system eq ""; 

    $result = system <<__END_OF_BPBNAT_ADDMACH_CALL__;
$bpnbat -addmachine  <<__END_OF_BPBNAT_ADDMACH_PARAMETERS__ > /dev/null
n
$at_broker_system
$at_broker_port
$client_system
$password
$password
__END_OF_BPBNAT_ADDMACH_PARAMETERS__
__END_OF_BPBNAT_ADDMACH_CALL__
    if ($result == 0) {
      logmsg("$step: added $client_system");
    } else {
      $fatal_errors++;
      logmsg("$step: ERROR: Call to 'bpnbat' command failed: $result");
    }
  }
}


################################################################################
#                                                                              #
# remote_system_type() - helper function use to sense platform idiosyncracies  #
#                        that complicate remote generation of shell scripts.   #
#                                                                              #
################################################################################
sub remote_system_type {
  my $remote_host = shift (@_);

  my $rsh_command = "rsh";
  my $platform_check = "uname";
  my $platform_type = "unknown";
  my $rsh_failed = 0;
  my @rsh_output = ();

  open(RSH, "$rsh_command $remote_host -l root $platform_check|") || ($rsh_failed = 1);
  if (! $rsh_failed) {
    @rsh_output = <RSH>;
    close(RSH);

    if ($#rsh_output >= 0) {
      chop($rsh_output[0]);
      return($rsh_output[0]);
    } else {
      return("VERITAS");
    }

  } else {
    return("VERITAS");
  }
}


################################################################################
#                                                                              #
# nbat_client_machine_login() - generate a shell script on each client system  #
#                               to be used to accomplish the machine login.    #
#                                                                              #
################################################################################
sub nbat_client_machine_login {
  my $echo_command = "/bin/echo";
  my $rsh_command = "rsh";
  my $bpnbat_call  = "$bpnbat -loginmachine ";

  my $remote_system_type = "VERITAS";

  my $build_remote_script = "";
  my $remote_script = "/tmp/VX__NBAC_HELPER_TEMP_FILE.tmp";
  my $secure_remote_script = "rm -f $remote_script";
  my $local_rsh_output = "/tmp/VX__NBAC_HELPER_OUT_FILE.tmp";
  my $result = -1;
  my $remote_result = "";
  my $ignored_result = "";

  my $client_system = "";
  my $user = "";
  my $password = "";

  my @failed_client_systems = ();


  logmsg("$step: Authenticating remote systems", "with VxSS.");

  foreach $client_system (@client_systems) {
    next if $client_system eq ""; 

    unlink "$local_rsh_output"; 

    logmsg("$step: contacting $client_system ...");

    $remote_system_type = &remote_system_type($client_system);
    if ($remote_system_type eq "Linux") {
      $echo_command = "/bin/echo -e";
    } else {
      $echo_command = "/bin/echo";
    }

    $password = $machine_password{$client_system} ? $machine_password{$client_system} : $default_machine_password;

    #
    # Protect shell meta characters in the password from expansion. 
    #
    # The substitution must be done twice here for a
    # a total of three times against the variable.
    # The second call below is not a typo.
    #
    $password =~ &replace_shell_meta_chars($password);
    $password =~ &replace_shell_meta_chars($password);

    $password =~ s/!/\\!/gx;

    $build_remote_script  = " $echo_command '\\043\\041/bin/sh' > $remote_script; ";
    $build_remote_script .= "chmod 0700 $remote_script; ";
    $build_remote_script .= "$echo_command '$bpnbat_call  > /dev/null <<__END_OF_BPNBAT_PARAMETERS__' >> $remote_script; ";
    $build_remote_script .= "$echo_command 'n' >> $remote_script; ";
    $build_remote_script .= "$echo_command '$at_broker_system' >> $remote_script; ";
    if ($at_broker_port eq "") {
      $build_remote_script .= "$echo_command >> $remote_script; ";
    } else {
      $build_remote_script .= "$echo_command '$at_broker_port' >> $remote_script; ";
    }
    $build_remote_script .= "$echo_command '$client_system' >> $remote_script; ";
    $build_remote_script .= "$echo_command '$password' >> $remote_script; ";
    $build_remote_script .= "$echo_command 'y' >> $remote_script; ";
    $build_remote_script .= "$echo_command '__END_OF_BPNBAT_PARAMETERS__' >> $remote_script; ";
    $build_remote_script .= "$echo_command '$echo_command \"REMOTE_RESULT=\\\044\\\077\"' >> $remote_script; ";

    #
    # Attempt generation of the remote shell script.  Ignore the result from rsh in
    # favor of a more useful result from the subsequent rsh call to execute the remote
    # script.
    #
    $result = system("$rsh_command $client_system -n -l root \"$build_remote_script\" ");

    #
    # Attempt to execute the remote script.  Search for the status value the script publicizes.
    #
    $result = system("$rsh_command $client_system -l root \"$remote_script\" > $local_rsh_output");
    if ($result == 0) {  # rsh returned success - transport did not fail

      open(RSH_OUT, "< $local_rsh_output") || ($result = -1);
      if ($result == 0) { # file open successful
        $remote_result = <RSH_OUT>;
        chop($remote_result);

        if ($remote_result =~ m/^REMOTE_RESULT=(\d+)/) { # remote script executed
          $result = $1;
        } else {
          $result = -1;
        }
        close(RSH_OUT);
      }
    }

    $ignored_result = unlink "$local_rsh_output";
    $ignored_result = system("$rsh_command $client_system -l root \"$secure_remote_script\" ");

    if ($result == 0) {
      logmsg("$step: $client_system login successful\n");
    } else {
      $fatal_errors++;
      logmsg("$step: ERROR: Call to 'bpnbat' command failed: $result on $client_system\n");

      push(@failed_client_systems, $client_system);
    }
  }

  if ($#failed_client_systems >= 0) {
    logmsg("$step: Errors were encountered attempting login ",
        "the following " . ($#failed_client_systems + 1) . " systems:");

    foreach $client_system (@failed_client_systems) {
      logmsg("$step: $client_system");
    }
    logmsg(" ");
    logmsg("Please review the preceeding log messages for the systems ",
        "involved for additional information.");
    logmsg(" ");
  }
}


################################################################################
# Driver logic                                                                 #
################################################################################

  &process_arguments;

  &list_preconditions;
  &prompt_for_continue;

  &prompt_for_mach_pass;
  &prompt_for_sec_admin_user;
  &prompt_for_sec_admin_pass;

  if ($at_broker_system eq "") {
    $at_broker_system = $master_system;
  }

  if ($az_server_system eq "") {
    $az_server_system = $master_system;
  }

  foreach $step (@rapid_install_steps) {
    print "-" x length($step), "\n\U$step\n", "-" x length($step), "\n";

    *subroutine = $step;
    &subroutine();

    if ($fatal_errors > 0) {
      die &$timestamp, "$step completed with fatal errors. Execution halted.\n";
    }

    print "\n";
  }


__END__
